<?php

namespace Drupal\azredirect\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\openid_connect\OpenIDConnectSession;
use Drupal\openid_connect\OpenIDConnectClaims;
use Drupal\openid_connect\Plugin\OpenIDConnectClientManager;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Psr\Log\LoggerInterface;

/**
 * AZRedirect event subscriber.
 */
class AzredirectSubscriber implements EventSubscriberInterface {

  /**
   * The OpenID Connect session service.
   *
   * @var \Drupal\openid_connect\OpenIDConnectSession
   */
  protected $session;

  /**
   * The OpenID Connect client manager.
   *
   * @var \Drupal\openid_connect\Plugin\OpenIDConnectClientManager
   */
  protected $pluginManager;

  /**
   * The OpenID Connect claims.
   *
   * @var \Drupal\openid_connect\OpenIDConnectClaims
   */
  protected $claims;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * The config factory (for editable config).
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $editableConfigFactory;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs the AZRedirect event subscriber.
   *
   * @param \Drupal\openid_connect\OpenIDConnectSession $session
   *   The OpenID Connect session service.
   * @param \Drupal\openid_connect\Plugin\OpenIDConnectClientManager $plugin_manager
   *   The OpenID Connect client manager.
   * @param \Drupal\openid_connect\OpenIDConnectClaims $claims
   *   The OpenID Connect claims.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route match.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $editable_config_factory
   *   The config factory for editable config.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(
    OpenIDConnectSession $session,
    OpenIDConnectClientManager $plugin_manager,
    OpenIDConnectClaims $claims,
    ConfigFactoryInterface $config_factory,
    AccountProxyInterface $current_user,
    CurrentRouteMatch $current_route_match,
    ConfigFactoryInterface $editable_config_factory,
    LoggerInterface $logger,
  ) {
    $this->session = $session;
    $this->pluginManager = $plugin_manager;
    $this->claims = $claims;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->currentRouteMatch = $current_route_match;
    $this->editableConfigFactory = $editable_config_factory;
    $this->logger = $logger;
  }

  /**
   * Register content type formats on the request object.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The Event to process.
   */
  public function onKernelRequest(RequestEvent $event) {
    if ($this->configFactory->get('azredirect.settings')->get('user_route')) {
      $route_name = $this->currentRouteMatch->getRouteName();
      if (
        $this->currentUser->isAnonymous() &&
        !in_array(
          $route_name,
          [
            'user.reset',
            'user.login.http',
            'user.login',
            'openid_connect.redirect_controller_redirect',
            'openid_connect_windows_aad.sso',
            'user.reset.login',
          ]
        )
      ) {
        $this->session->saveDestination();
        // Use dependency injection for logger and config factory.
        $this->logger->notice($route_name);
        $client_name = 'windows_aad';
        $configuration = $this->editableConfigFactory
          ->getEditable('openid_connect.client.' . $client_name)
          ->get('settings');
        /** @var \Drupal\openid_connect\Plugin\OpenIDConnectClientInterface $client */
        $client = $this->pluginManager->createInstance(
          $client_name,
          $configuration
        );
        $client->setParentEntityId($client_name);
        $scopes = $this->claims->getScopes($client);
        $_SESSION['openid_connect_op'] = 'login';
        $response = $client->authorize($scopes);
        $event->setResponse($response);
      }
    }
  }

  /**
   * Kernel response event handler.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The Response event.
   */
  public function onKernelResponse(ResponseEvent $event) {
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => ['onKernelRequest'],
      KernelEvents::RESPONSE => ['onKernelResponse'],
    ];
  }

}
